#!/usr/bin/perl -w

# mklinkpg.pl - Creates a page of links based on a template.
# Originally written 11/18/2000, A. James Yolkowski (ajy@sentex.net).
# Updated documentation, 12/10/2000.

=head1 NAME

mklinkpg.pl - Make a links page from a list of URLs and a template

=head1 SYNOPSIS

mklinkpg.pl C<-t> I<template_file> [I<links_file>]

=head1 DESCRIPTION

mklinkpg.pl creates a links page given a web page template and a
list of URLs to include in the web page.  It does this by fetching
each URL and extracting the page title and description from the
page.  mklinkpg.pl writes its output to standard output.  Unless
I<links_file> is specified, mklinkpg.pl reads the list of links from
standard input.

I<links_file> should contain a list of URLs, one per line.  Any text
after the URL will be interpreted as the title to use for the page.
If this is specified, mklinkpg.pl will not attempt to fetch the page.
If any line in I<links_file> does not look like it starts with a protocol,
it is interpreted as a section name.

I<template_file> should be a template HTML file that uses the following
special tokens:

=over 4

=item *

<!--@START--> : HTML code to insert at the start of the page.

=item *

<!--@END--> : HTML code to insert at the end of the page.

=item *

<!--@SECTION_START--> : HTML code to insert at the start of each
section; it may contain the special token **SECTION**, which will
be replaced by the name of the section.

=item *

<!--@SECTION_END--> : HTML code to insert at the end of a section.

=item *

<!--@TEMPLATE--> : The HTML template to use for each link.  It
may contain the special tokens **URL**, **TITLE**, and **DESC**,
which will be replaced with the URL of the page, the title of the
page, and the description of the page, respectively.

=back

=head1 AUTHOR

James Yolkowski (ajy@sentex.net).

=head1 COPYRIGHT and LICENCE

This program is Copyright (C) 2000, James Yolkowski

This program is free and open software. You may use, modify, distribute
and sell this program (and any modified variants) in any way you wish,
provided you do not restrict others from doing the same.

=cut

use HTML::Parser;
use URI::URL;
use LWP::UserAgent;
use Getopt::Std;

package GetTitle;

@ISA=qw/HTML::Parser/;
my ($title, $description);
my $in_title = undef;

sub init {
    $title = '';
    $description = '';
}

sub start {
    my ($self, $tag, $attr) = @_;
    if ('title' eq lc $tag) {
	$title = '';
	$in_title = 1;
    }
    elsif (lc($tag) eq 'meta' and exists $attr->{'name'}
	   and lc($attr->{'name'} eq 'description')) {
	$description = $attr->{'content'};
    }
}

sub end {
    my ($self, $tag) = @_;
    undef $in_title if 'title' eq lc $tag;
}

sub text {
    my ($self, $text) = @_;
    $title .= $text if $in_title;
}

sub get_title{$title}
sub get_desc{$description}

package main;
%urls = ();
@urls = ();
%templates = ();

getopts('t:');
die "Template file not specified.\n" unless $opt_t;

# Read the list of links.
while(<>) {
    if (/^\w+:/) {		# Look for a protocol.
	my ($url, $desc) = (/(\S+)\s*(.*)/);
	$urls{$url} = ($desc or undef);
	push @urls, $url;
    }
    else {
	chomp;
	push @urls, $_;
    }
}

# Read the template file.
open TEMPLATE, "$opt_t";
{
    undef $/;
    $_ = <TEMPLATE>;
# Parse the template file to obtain the following sections:
# START, END, SECTION_START, SECTION_END, TEMPLATE.
# For now, all of these are required.
# In the future, perhaps also have PARAMETERS and LINKS sections.
    (undef, %templates) = split /^<!--@(.*?)-->\n/m;
}
close TEMPLATE;

print $templates{START};
my $first = 1;
my $section = undef;

for (@urls) {
    if (not /^\w+:/) {
	if ($first) {undef $first}
	else {print $templates{SECTION_END}}
	my $template = $templates{SECTION_START};
	$template =~ s/\*\*SECTION\*\*/$_/g;
	print $template;
	$section = 1;
    }
    elsif (not $urls{$_}) {
	my $useragent = LWP::UserAgent->new;
	my $url = $_;
	my $response = $useragent->request(HTTP::Request->new("GET", $url));
	my $newurl = $response->request->url;

	# Print debugging data if something happened.
	print STDERR "Note: Redirected to $newurl from $url\n"
	    if $url ne $newurl;
	if ($response->code != 200) {
	    print STDERR "Error ", $response->code, " from $_\n";
	}
	else {
	    # Parse the HTML returned to get the page title.
	    my $parser = new GetTitle;
	    $parser->init;
	    $parser->parse($response->content);
	    my $title = $parser->get_title;
	    my $long_description = $parser->get_desc;

	    my $template = $templates{TEMPLATE};
	    $template =~ s/\*\*URL\*\*/$newurl/g;
	    $template =~ s/\*\*TITLE\*\*/$title/g;
	    $template =~ s/\*\*DESC\*\*/$long_description/g;
	    print $template;
	}
    }
    else {
	my $template = $templates{TEMPLATE};
	my $description = $urls{$_};
	$template =~ s/\*\*URL\*\*/$_/g;
	$template =~ s/\*\*TITLE\*\*/$description/g;
	$template =~ s/\*\*DESC\*\*//g;	# Description is blanked out!
	print $template;
    }
}

print $templates{SECTION_END} if $section;
print $templates{'END'};
